function [x_bulk, bulk, mode] = clusterize(data)
% PURPOSE: Clusterize the data
% ----------------------------------------------------
% USAGE:   [x_bulk, bulk, mode] = clusterize(data)
% where:   data = d x n-matrix of the original n d-dimensional vectors
% -----------------------------------------------------
% RETURNS: x_bulk = d x m-matrix of points in the bulk
%          bulk   = index vector of the bulk points
%          mode   = mode of the biggest cluster as a column vector
% -----------------------------------------------------    

    data_tmp = data;

    [d, n] = size(data);

    % Compute the bandwidth
    [H2, h, center] = bandwidth2(data);
    
    % Transform the data using the covariance part of the bandwidth
    [V, E] = eig(H2);
    H  = V*sqrt(E)*inv(V);
    iH = inv(H);
    
    for i = 1:n
        data(:, i) = iH*(data(:, i) - center);
    end
    
    %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
    %                                  HMAC                               %
    %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
    
    % Perform HMAC using h as a scalar bandwidth
    modes = zeros(d, n);

    for i = 1:n
        x0     = data(:, i)';
        x0_old = x0 + 1;
        
        cnt = 0;
        err = 100;
        
        while (err > 1E-4) && (cnt < 150)
            f = mvnpdf(x0, data', h*eye(d));
            
            p  = f/sum(f);
            x0 = p'*data';
            
            err = norm(x0_old - x0)/max(norm(x0), 1);
            
            x0_old = x0;
            
            cnt = cnt + 1;
        end
        
        modes(:, i) = x0';
    end
  
    clusters  = zeros(1, n);
    clust_cnt = 0;
    
    err = 1/(2*n);
    for i = 1:n
        if (clusters(i) == 0)
            clust_cnt   = clust_cnt + 1;
            clusters(i) = clust_cnt;
            
            for j = 1:n
                if (clusters(j) == 0)
                    if norm(modes(:, i) - modes(:, j)) < err
                        clusters(j) = clust_cnt;
                    end
                end
            end
        end
    end
    
    clust_i = -1;
    clust_s = 0;
    for i = 1:clust_cnt
        s = length(find(clusters == i));
        
        if (s > clust_s)
            clust_i = i;
            clust_s = s;
            
            ind  = min(find(clusters == i));
            mode = modes(:, ind);
        end
    end

    bulk = find(clusters == clust_i);

    % Compute the symmetry measure
    sm = symmetry_measure(bulk);
    
    % Perform a symmetry screening
    threshold = symmetry_threshold(0.05, d, n);
    if (threshold >= 0)
        [ssm, indsm] = sort(sm, 'ascend');
        
        inliers  = find(sm >= threshold);
        outliers = find(sm <  threshold);

        scale = length(inliers)/length(bulk);
        bulk = bulk(inliers);
    end

    data = data_tmp;

    x_bulk = data(:, bulk);
    mode = H*mode + center; 
    
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
    function f = density(x)
        f = mvnpdf(x', data', h*eye(d));
        f = 1/n*sum(f);
    end

    function sm = symmetry_measure(cluster)
        sm = zeros(size(cluster));
        
        for i = 1:length(cluster)
            x1 = data(:, cluster(i));
            x2 = 2*mode - x1;
            f1 = density(x1);
            f2 = density(x2);
            sm(i) = min(f1, f2)/max(f1, f2);
        end
    end 
end