function [limit, runtime, S, gamma_est] = compute_limits(d, n, method, sample_cnt, gamma)
% PURPOSE: Compute the family-wise and per-comparison limits
% ----------------------------------------------------
% USAGE:   [limit, runtime, S, gamma_est] = compute_limits(d, n, method, sample_cnt, gamma)
% where:   d = dimension
%          n = sample size
%          method = 0 (IRMCD with asymptotic limits described by Cerioli (2010)
%                   1 (IRMCD with limits based on simulation)
%                   2 (FSRMCD-MAC with limits based on simulation)
%          sample_cnt = number of samples to take to simulate the limits
%          gamma = a two-component test size vector (family-wise and
%                  per-comparison sizes), nominal for method = 0, actual
%                  for method = 1, 2
% -----------------------------------------------------
% RETURNS: limit = [], of method 0
%                  limit.m, limit.r, if method = 1, 2
%                  limits.m - a three-component vector with family-wise
%                  limits for points with weights w_i = 0, w_i = 1, w_i
%                  arbtitrary
%                  limits.r - a three-component vector with per-comparison
%                  limits for points with weights w_i = 0, w_i = 1, w_i
%                  arbtitrary
%          S = average of covariance matrix estimates produced
%          gamma_est = estimated (family-wise) test size
% -----------------------------------------------------    

    S = zeros(d, d);

    switch (method)
        case 0
            limit = [];
            
            runtime = 0;
            cnt = 0;
            for i = 1:sample_cnt
                x = randn(d, n);

                tic;
                output = compute_signals(x, method, gamma);
                runtime = runtime + toc;
                
                S = S + output.cov/sample_cnt;
                sig = output.signals;
                
                if (~isempty(output.outliers))
                    cnt = cnt + 1;
                end
            end
            runtime = runtime/sample_cnt;
            
            gamma_est = cnt/sample_cnt;
            
        otherwise
            runtime = 0;
            sig_m0 = [];
            sig_m1 = [];
            sig_r0 = [];
            sig_r1 = [];
            sig_m = [];
            sig_r = [];
            
            for i = 1:sample_cnt
                x = randn(d, n);

                tic;
                output = compute_signals(x, method, gamma);
                runtime = runtime + toc;
                
                S = S + output.cov/sample_cnt;
                sig = output.signals;
                
                %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
                % Weight 0
                %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
                I = find(output.weights == 0);
                if (~isempty(I))                   
                    ind = randi(length(I), 1);

                    sig_m0 = [sig_m0 max(sig(I))];
                    sig_r0 = [sig_r0 sig(I(ind))];
                end
                
                %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
                % Weight 1
                %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
                I = find(output.weights == 1);
                if (~isempty(I)) 
                    ind = randi(length(I), 1);
                
                    sig_m1 = [sig_m1 max(sig(I))];
                    sig_r1 = [sig_r1 sig(I(ind))];
                end
                
                %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
                % Overall
                %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
                ind = randi(n, 1);
                sig_m = [sig_m max(sig)];
                sig_r = [sig_r sig(ind)];
            end
            
            runtime = runtime/sample_cnt;

            limit.m(1) = quantile(sig_m0, 1 - gamma);
            limit.m(2) = quantile(sig_m1, 1 - gamma);
            limit.m(3) = quantile(sig_m,  1 - gamma);
            
            limit.r(1) = quantile(sig_r0, 1 - gamma);
            limit.r(2) = quantile(sig_r1, 1 - gamma);
            limit.r(3) = quantile(sig_r,  1 - gamma);
            
            gamma_est = gamma;
    end
end