function [rate, runtime] = compute_method_power(d, n, method, gamma, sample_cnt, k, cont_type, cont_severity, limit)
% PURPOSE: Compute detection power
% ----------------------------------------------------
% USAGE:   compute_discrimination_rate(d, n, method, gamma, sample_cnt, k, cont_type, cont_severity, limit)
% where:   d = dimension
%          n = sample size
%          method = 0 (IRMCD with asymptotic limits described by Cerioli (2010)
%                   1 (IRMCD with limits based on simulation)
%                   2 (FSRMCD-MAC with limits based on simulation)
%          gamma = a two-component test size vector (family-wise and
%                  per-comparison sizes), nominal for method = 0, actual
%                  for method = 1, 2
%          sample_cnt = number of samples to take to simulate the the
%                       detection powers
%          k = vector containing number of outliers in each scenario
%          cont_type = contamination type:
%                    = 0 (shift lambda in mean)
%                    = 1 (increased scattering psi)
%                    = 2 (polution as t-distribution zeta)
%          cont_severity = vector of severity measure for contaminination
%          limit = consisting of
%                  limit.m = a three component vector containing
%                  family-wise limits for points with weight 0, weight 1
%                  and arbitrary weight
%                  limit.r = a three component vector containing
%                  per comparison limits for points with weight 0, weight 1
%                  and arbitrary weight
% -----------------------------------------------------
% RETURNS: rate = consisting of
%                 rate.power = detection power
%                 rate.error = swamping rate
%          runtime = runtime for the whole scenario (in sec)
% -----------------------------------------------------    

    switch (method)
    	case 0
            rate.power = 0;
            rate.error = 0;
        otherwise
            rate.power = zeros(2, 1);
            rate.error = zeros(2, 1);
    end
    
    runtime = 0;
    for i = 1:sample_cnt
        x = randn(d, n);
        
        x = randn(d, n);
        p = randperm(n);
        
        % Change in mean
        if (cont_type == 0)
            mu = ones(d, 1)*cont_severity;
             
            for j = 1:k
                x(:, p(j)) = x(:, p(j)) + mu;
            end
        % Change in variance
        elseif (cont_type == 1)
            for j = 1:k
                x(:, p(j)) = x(:, p(j))*sqrt(cont_severity);
            end
        % Change in shape
        else
            for j = 1:k
                u = chi2rnd(cont_severity);
                x(:, p(j)) = x(:, p(j))*sqrt(cont_severity/u);
            end
        end
            
        tic;
        output  = compute_signals(x, method, gamma);
        runtime = runtime + toc;
        
        switch (method)
            case 0
                rate.power = rate.power + length(intersect(output.outliers, p(1:k)))/k;
                rate.error = rate.error + length(intersect(output.outliers, p(k+1:n)))/(n - k);
                
            otherwise 
                %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
                %
                %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
                I0 = find(output.weights == 0);
                I1 = find(output.weights == 1);

                if (~isempty(find(output.signals(I0) > limit.m(1))) || ~isempty(find(output.signals(I0) > limit.m(2))))
                    I0p = intersect(I0, p(1:k));
                    I1p = intersect(I1, p(1:k));
                    I0e = intersect(I0, p(k+1:n));
                    I1e = intersect(I1, p(k+1:n));

                    rate.power(1) = rate.power(1) ...
                                  + (length(find(output.signals(I0p) > limit.r(1))) + length(find(output.signals(I1p) > limit.r(2))))/k;
                    rate.error(1) = rate.error(1) ...
                                  + (length(find(output.signals(I0e) > limit.r(1))) + length(find(output.signals(I1e) > limit.r(2))))/(n - k);
                end

                %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
                %
                %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
                if (length(find(output.signals > limit.m(3))) > 0)
                    rate.power(2) = rate.power(2) + length(find(output.signals(p(1:k)) > limit.r(3)))/k;
                    rate.error(2) = rate.error(2) + length(find(output.signals(p(k+1:n)) > limit.r(3)))/(n - k);
                end
        end
    end
    
    runtime = runtime/sample_cnt;
    
    rate.power = rate.power/sample_cnt;    
    rate.error = rate.error/sample_cnt;  
end