function output = compute_signals(x, method, gamma)
% PURPOSE: Compute the outlier detection signals
% ----------------------------------------------------
% USAGE:   sig = compute_signals(x, threshold, disp_flag)
% where:   x = d x n-matrix of the original n d-dimensional vectors
%          method = 0 (IRMCD with asymptotic limits described by Cerioli (2010)
%                   1 (IRMCD with limits based on simulation)
%                   2 (FSRMCD-MAC with limits based on simulation)
%          gamma = a two-component test size vector (family-wise and
%                  per-comparison sizes), nominal for method = 0, actual
%                  for method = 1, 2
% -----------------------------------------------------
% RETURNS: output containing 
%          output.signals  = outlier detection signals
%          output.weights  = weight w_i used to compute the estimates
%          output.outliers = outliers find by IRMCD (if method = 0)
%          output.loc      = estimated mean vector
%          output.cov      = estimated covariance
% -----------------------------------------------------    

    [d, n] = size(x);

    output.method = method;

    switch (method)
        case 0
            [RAW, REW, IR] = irmcd(x', 'msg', 0, 'betathresh', 1, 'conflev', (1 - gamma)^(1/n), 'conflevrew', 0.975, ...
                                   'nsamp', 500, 'refsteps', 20, 'reftol', 1e-10, 'refstepsbestr', 20);        
            output.signals  = REW.md;
            output.weights  = RAW.weights;
            output.outliers = IR.outliers;
            output.loc = REW.loc;
            output.cov = REW.cov;
        case 1
            [RAW, REW, IR] = irmcd(x', 'msg', 0, 'betathresh', 1, 'conflev', (1 - gamma)^(1/n), 'conflevrew', 0.975, ...
                             'nsamp', 500, 'refsteps', 20, 'reftol', 1e-10, 'refstepsbestr', 20);      
            output.signals  = REW.md;
            output.weights  = RAW.weights;
            output.loc = REW.loc;
            output.cov = REW.cov;
        otherwise
            %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
            %                                Clustering                               %
            %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
            [x_bulk, bulk, mode] = clusterize(x);       

            if (size(x_bulk, 2) < d + 1)
                x_bulk = x;
                mode = mean(x_bulk, 2);
            end

            %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
            %                               Reweightening                             %                                                                                                                                               %
            %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%    
            output.signals = zeros(1, n);
            
            c  = mean(x_bulk, 2);
            S  = cov(x_bulk');

            iS = inv(S);

            q = chi2inv(0.975, d);

            c_rew = zeros(d, 1);
            m = 0;
            for i = 1:n
                T2 = (x(:, i) - c)'*iS*(x(:, i) - c);

                if (T2 <= q)
                    m = m + 1;
                    c_rew = c_rew + x(:, i);
                    output.weights(i) = 1;
                else
                    output.weights(i) = 0;
                end
            end
            c_rew = c_rew/m;
       
            S_rew = zeros(d, d);
            m = 0;
            for i = 1:n
                T2 = (x(:, i) - c)'*iS*(x(:, i) - c);

                if (T2 <= q)
                    m = m + 1;
                    S_rew = S_rew + (x(:, i) - c_rew)*(x(:, i) - c_rew)';
                end
            end   
            S_rew = S_rew/(m - 1);

            %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
            %                            Signal Computation                           %                                                                                                                                               %
            %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
            output.signals = zeros(1, n);

            c  = c_rew;
            S  = S_rew;
            
            output.loc = c;
            output.cov = S;

            iS = inv(S);

            for i = 1:n
                T2 = (x(:, i) - c)'*iS*(x(:, i) - c);
                output.signals(i) = T2;
            end
    end
end