function make_report(dim, sample_size, method, gamma, outliers, cont_type, cont_severity, sample_cnt, path)
% PURPOSE: Make simulation report
% ----------------------------------------------------
% USAGE:   make_report(dim, sample_size, method, gamma, outliers, cont_type, cont_severity, sample_cnt, path)
% where:   dim = dimension
%          sample_size = sample size
%          method = 0 (IRMCD with asymptotic limits described by Cerioli (2010)
%                   1 (IRMCD with limits based on simulation)
%                   2 (FSRMCD-MAC with limits based on simulation)
%          gamma = a two-component test size vector (family-wise and
%                  per-comparison sizes), nominal for method = 0, actual
%                  for method = 1, 2
%          outliers = vector containing number of outliers in each scenario
%          cont_type = contamination type:
%                    = 0 (shift lambda in mean)
%                    = 1 (increased scattering psi)
%                    = 2 (polution as t-distribution zeta)
%          cont_severity = vector of severity measure for contaminination
%          sample_cnt = a two-component vector with two integers determining the
%                       number of samples to take to simulate the limits and
%                       detection rates
%          path = output path
% -----------------------------------------------------
% RETURNS: []
% -----------------------------------------------------    

    if ~exist('output', 'dir')
        mkdir('output');
    end

    if ~exist('output/report', 'dir')
        mkdir('output/report');
    end
   
    % Change in mean
    if (cont_type == 0)
        cont_str = 'lambda';
    % Change in variance
    elseif (cont_type == 1)
        cont_str = 'psi';
    % Change in shape
    else
        cont_str = 'zeta';
    end 

    display(['Computing: dim = ', int2str(dim), ', n = ', int2str(sample_size), ', gamma = ', num2str(gamma), ' ...']);

    switch (method)
        case 0
        name = [path int2str(dim) 'd.' int2str(sample_size) 'pts.gamma=' num2str(gamma) '.' cont_str '.IRMCD-asymptotical.report'];   
        
        case 1
        name = [path int2str(dim) 'd.' int2str(sample_size) 'pts.gamma=' num2str(gamma) '.' cont_str '.IRMCD-empirical.report']; 
        
        case 2
        name = [path int2str(dim) 'd.' int2str(sample_size) 'pts.gamma=' num2str(gamma) '.' cont_str '.FSRMCD-MAC-empirical.report'];
    end
    
    file_id = fopen([name '.csv'], 'w'); 

    [limit, avruntime, S, gamma_est] = compute_limits(dim, sample_size, method, sample_cnt(1), gamma);
    
    fprintf(file_id, 'Dimension: d; %d; \n', dim);
    fprintf(file_id, 'Number of points in the sample: n; %d; \n', sample_size);
    fprintf(file_id, 'Sample number for quantiles: N1; %d; \n', sample_cnt(1));
    fprintf(file_id, 'Sample number for signal probability: N2; %d; \n', sample_cnt(2));
    %
    if (method > 0)
        fprintf(file_id, 'gamma; %8.5f; \n', gamma);
        %%%%%%%%%%
        fprintf(file_id, '\n\n');
        fprintf(file_id, 'CONDITIONING ON W\n');
        fprintf(file_id, 'Weight = 0: ; \n');
        fprintf(file_id, 'Control limit; %8.5f; \n', limit.m(1));
        fprintf(file_id, '(1 - gamma)-th quantile; %8.5f; \n', limit.r(1));
        fprintf(file_id, 'Weight = 1: ; \n');
        fprintf(file_id, 'Control limit; %8.5f; \n', limit.m(2));
        fprintf(file_id, '(1 - gamma)-th quantile; %8.5f; \n', limit.r(2));
        %%%%%%%%%%
        fprintf(file_id, '\n');
        fprintf(file_id, 'NO CONDITIONING ON W\n');
        fprintf(file_id, 'Control limit; %8.5f; \n', limit.m(3));
        fprintf(file_id, '(1 - gamma)-th quantile; %8.5f; \n', limit.r(3));
    else
        fprintf(file_id, 'Nominal gamma; %8.5f; \n', gamma);
        fprintf(file_id, 'Estimated gamma; %8.5f; \n', gamma_est);
    end
    fprintf(file_id, '\n\n\n');

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

    runtimes = zeros(length(outliers), length(cont_severity));
    
    rate.power = zeros(length(outliers), length(cont_severity));
    rate.error = zeros(length(outliers), length(cont_severity));
    
    rate_c.power = zeros(length(outliers), length(cont_severity));
    rate_c.error = zeros(length(outliers), length(cont_severity));
    
    rate_nc.power = zeros(length(outliers), length(cont_severity));
    rate_nc.error = zeros(length(outliers), length(cont_severity));
            
    cnt = 0;        
    
    for j = 1:length(outliers)
        for k = 1:length(cont_severity)
            cnt = cnt + 1;
   
            display(['Scenario no. ', int2str(cnt), ' out of ', int2str(length(outliers)*length(cont_severity)), ...
                     ': k = ', int2str(outliers(j)), ', ', cont_str, ' = ', num2str(cont_severity(k))]);
            [rate_tmp, runtime] = compute_method_power(dim, sample_size, method, gamma, sample_cnt(2), outliers(j),  cont_type, cont_severity(k), limit);
            
            if (method > 0)
                rate_c.power(j, k) = rate_tmp.power(1);
                rate_c.error(j, k) = rate_tmp.error(1);
                
                rate_nc.power(j, k) = rate_tmp.power(2);
                rate_nc.error(j, k) = rate_tmp.error(2);
            else
                rate.power(j, k) = rate_tmp.power(1);
                rate.error(j, k) = rate_tmp.error(1);               
            end
            
            runtimes(j, k) = runtime;
        end
    end
    
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
        
    if (method > 0)
        %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
        % Conditioning on w
        %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
        fprintf(file_id, 'CONDITIONING ON W\n');
        fprintf(file_id, 'Power of the test: \n');
        fprintf(file_id, ['k\\', cont_str, '; ']); 
        for k = 1:length(cont_severity)
            fprintf(file_id, '%8.5f; ', cont_severity(k));
        end
        fprintf(file_id, '\n');
        for j = 1:length(outliers)
            fprintf(file_id, '%d; ', outliers(j));
            for k = 1:length(cont_severity) 
                fprintf(file_id, '%8.5f; ', rate_c.power(j, k));
            end
            fprintf(file_id, '\n');
        end
        fprintf(file_id, '\n');
        %%%%%%
        fprintf(file_id, 'Error rate: \n');
        fprintf(file_id, ['k\\', cont_str, '; ']);
        for k = 1:length(cont_severity)
            fprintf(file_id, '%8.5f; ', cont_severity(k));
        end
        fprintf(file_id, '\n');
        for j = 1:length(outliers)
            fprintf(file_id, '%d; ', outliers(j));
            for k = 1:length(cont_severity) 
                fprintf(file_id, '%8.5f; ', rate_c.error(j, k));
            end
            fprintf(file_id, '\n');
        end
        fprintf(file_id, '\n\n');
        %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
        % No conditioning on w
        %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
        fprintf(file_id, 'NO CONDITIONING ON W\n');
        fprintf(file_id, 'Power of the test: \n');
        fprintf(file_id, ['k\\', cont_str, '; ']);
        for k = 1:length(cont_severity)
            fprintf(file_id, '%8.5f; ', cont_severity(k));
        end
        fprintf(file_id, '\n');
        for j = 1:length(outliers)
            fprintf(file_id, '%d; ', outliers(j));
            for k = 1:length(cont_severity) 
                fprintf(file_id, '%8.5f; ', rate_nc.power(j, k));
            end
            fprintf(file_id, '\n');
        end
        fprintf(file_id, '\n');
        %%%%%%
        fprintf(file_id, 'Error rate: \n');
        fprintf(file_id, ['k\\', cont_str, '; ']);
        for k = 1:length(cont_severity)
            fprintf(file_id, '%8.5f; ', cont_severity(k));
        end
        fprintf(file_id, '\n');
        for j = 1:length(outliers)
            fprintf(file_id, '%d; ', outliers(j));
            for k = 1:length(cont_severity) 
                fprintf(file_id, '%8.5f; ', rate_nc.error(j, k));
            end
            fprintf(file_id, '\n');
        end
        fprintf(file_id, '\n\n');    
    else
        fprintf(file_id, 'CONDITIONING ON W\n');
        fprintf(file_id, 'Power of the test: \n');
        fprintf(file_id, ['k\\', cont_str, '; ']);
        for k = 1:length(cont_severity)
            fprintf(file_id, '%8.5f; ', cont_severity(k));
        end
        fprintf(file_id, '\n');
        for j = 1:length(outliers)
            fprintf(file_id, '%d; ', outliers(j));
            for k = 1:length(cont_severity) 
                fprintf(file_id, '%8.5f; ', rate.power(j, k));
            end
            fprintf(file_id, '\n');
        end
        fprintf(file_id, '\n');
        %%%%%%
        fprintf(file_id, 'Error rate: \n');
        fprintf(file_id, ['k\\', cont_str, '; ']);
        for k = 1:length(cont_severity)
            fprintf(file_id, '%8.5f; ', cont_severity(k));
        end
        fprintf(file_id, '\n');
        for j = 1:length(outliers)
            fprintf(file_id, '%d; ', outliers(j));
            for k = 1:length(cont_severity) 
                fprintf(file_id, '%8.5f; ', rate.error(j, k));
            end
            fprintf(file_id, '\n');
        end
        fprintf(file_id, '\n\n');        
    end
    
    %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%    
    if (method > 0)
        fprintf(file_id, 'Mean eigenvalue of the mean covariance:; %8.5f \n', mean(eig(S)));
        fprintf(file_id, 'Mean covariance: \n');
        for i = 1:dim
            fprintf(file_id, '; ');
            for j = 1:dim
                fprintf(file_id, '%8.5f; ', S(i, j));
            end
            fprintf(file_id, '\n');
        end
    end
    
    fprintf(file_id, '\n\nAverage runtimes\n\n');
    fprintf(file_id, ['k\\', cont_str, '; ']);
    for k = 1:length(cont_severity)
        fprintf(file_id, '%8.5f; ', cont_severity(k));
    end        
    fprintf(file_id, '\n');
    for j = 1:length(outliers)
        fprintf(file_id, '%d; ', outliers(j));

        for k = 1:length(cont_severity)
            fprintf(file_id, '%8.5f; ', runtimes(j, k));
        end

        fprintf(file_id, '\n');
    end        

    fprintf(file_id, '\nMaximum average runtime; %8.5f', max(max(max(runtimes)), avruntime));

    fclose(file_id);
end